mcClust, mcClustExtended,
dmClust or dmClustExtended as well as MNLAuxMix.calcAllocationsMCC(outList, thin = 1, maxi = 50,
M0 = outList$Mcmc$M0, plotPathsForEta = TRUE)
calcAllocationsMCCExt(outList, thin = 1, maxi = 50,
M0 = outList$Mcmc$M0)
calcAllocationsDMC(outList, thin = 1, maxi = 50,
M0 = outList$Mcmc$M0, plotPathsForEta = TRUE)
calcAllocationsDMCExt(outList, thin = 1, maxi = 50,
M0 = outList$Mcmc$M0)
calcAllocationsMNL(outList, thin = 1, maxi = 50,
M0 = outList$Mcmc$M0)estGroupSize contains the mixing proportions/weights $\hat{\eta}$. In these cases
each thin-th MCMC draw beginning at M0 (after burn-in) is used for calculation. For MCC and DMC
with MNL extension and MNLAuxMix the group sizes are calculated based on the individual posterior
classification probabilities which are calculated using the last maxi draws of each thin-th MCMC draw.maxi draws of each thin-th MCMC draw.maxi MCMC draws of each thin-th draw are taken for calculations, except for mixing
proportions $\eta$ (which are part of MCC and DMC without MNL extension) where all thin-th
draws beginning at M0 are used.mcClust, dmClust, mcClustExtended, dmClustExtended,
MNLAuxMix# please run the examples in mcClust, dmClust, mcClustExtended,
# dmClustExtended, MNLAuxMixRun the code above in your browser using DataLab